package util;

import org.xml.sax.SAXException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dom4j.DocumentException;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.net.URL;
import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import java.util.Date;
import java.util.Iterator;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import com.meterware.httpunit.*;
import server.Request;

public class LastFmLookup {

    private static Log LOG = LogFactory.getLog(LastFmLookup.class);

    public static final String ERROR_BIO_NOT_FOUND = "band info not found";
    public static final String ERROR_NOT_FRIENDS_NOT_FOUND = "top friends not found";

    public static String getBio(String artist) throws IOException, SAXException {
        WebConversation wc = new WebConversation();
        WebRequest req = new GetMethodWebRequest("http://www.last.fm/music/" + URLEncoder.encode(artist, "UTF-8") + "/+wiki");
        WebResponse resp = wc.getResponse(req);
        String text = resp.getText();
        int i = text.indexOf("View wiki recent changes page");
        if (i == -1) {
            throw new RuntimeException(ERROR_BIO_NOT_FOUND);
        }
        text = text.substring(i);
        String startSearchString = "</div>";
        String endSearchString = "<div style";
        int startBioIndex = text.indexOf(startSearchString) + startSearchString.length();
        if (startBioIndex == -1) {
            throw new RuntimeException(ERROR_BIO_NOT_FOUND);
        }
        int endBioIndex = text.indexOf(endSearchString, startBioIndex);
        if (endBioIndex == -1) {
            throw new RuntimeException(ERROR_BIO_NOT_FOUND);
        }
        return Util.addLastFmUrl(text.substring(startBioIndex, endBioIndex));
    }

    public static String getTopFans(String artist, String song) throws IOException, SAXException {
        LOG.info("looking up top fans");
        WebConversation wc = new WebConversation();
        String s = "http://www.last.fm/music/" + URLEncoder.encode(artist, "UTF-8") + "/_/" + URLEncoder.encode(song, "UTF-8");
        WebRequest req = new GetMethodWebRequest(s);
        WebResponse resp = wc.getResponse(req);
        String text = resp.getText();
        String startSearch = "c_fanspanel\" >";
        int startTopArtistIndex = text.indexOf(startSearch);
        if (startTopArtistIndex == -1) {
            throw new RuntimeException(ERROR_NOT_FRIENDS_NOT_FOUND);
        }
        int endTopArtistIndex = text.indexOf("<span class=\"mozillasucks\">", startTopArtistIndex);
        return Util.addLastFmUrl(text.substring(startTopArtistIndex + startSearch.length(), endTopArtistIndex));
    }

    public static LastFmTrackStats getTrackStat(String artist, String song) throws IOException, SAXException {
        LastFmTrackStats lastFmTrackStats = new LastFmTrackStats();

        WebConversation wc = new WebConversation();
        HttpUnitOptions.setScriptingEnabled(false);
        String s = "http://www.last.fm/music/" + URLEncoder.encode(artist, "UTF-8") + "/_/" + URLEncoder.encode(song, "UTF-8");
        WebRequest req = new GetMethodWebRequest(s);
        LOG.info("fetching stats: " + s);
        WebResponse resp = wc.getResponse(req);
        String text = resp.getText();

        int endStatIndex = setTrackStats(text, lastFmTrackStats);
        int endImageIndex = setAlbumImageLocation(text, endStatIndex, lastFmTrackStats);
        setSimilarArtist(text, endImageIndex, lastFmTrackStats);

        return lastFmTrackStats;
    }

    private static void setSimilarArtist(String text, int endImageIndex, LastFmTrackStats lastFmTrackStats) {
        String startSimilar = "c_simpanel\" >";
        int startSimilarIndex = text.indexOf(startSimilar, endImageIndex) + startSimilar.length();
        int endSimilarIndex = text.indexOf("</div>", startSimilarIndex);
        lastFmTrackStats.setSimilarArtist("<span>Similar Artists:</span> <br/>" + Util.addLastFmUrl(text.substring(startSimilarIndex, endSimilarIndex).trim()));
    }

    private static int setAlbumImageLocation(String text, int endStatIndex, LastFmTrackStats lastFmTrackStats) {
        int startImageIndex = text.indexOf("http://static.last.fm/coverart/", endStatIndex);
        int endImageIndex;
        if (startImageIndex == -1) {
            startImageIndex = text.indexOf("http://images.amazon.com/images/", endStatIndex);
            if (startImageIndex == -1) {
                endImageIndex = endStatIndex;
                LOG.info("no image found");
            } else {
                endImageIndex = getImgLoc(text, lastFmTrackStats, startImageIndex, true);
            }
        } else {
            endImageIndex = getImgLoc(text, lastFmTrackStats, startImageIndex, false);
        }
        return endImageIndex;
    }

    private static int setTrackStats(String text, LastFmTrackStats lastFmTrackStats) {
        int startTracjInfo = text.indexOf("Track Statistics");
        if (startTracjInfo == -1) {
            throw new RuntimeException("Song info not found");
        }

        String endSearch = "</p>";

        int startStatIndex = text.indexOf("<p>", startTracjInfo);
        int endStatIndex = text.indexOf(endSearch, startStatIndex);
        lastFmTrackStats.setTrackStats(Util.addLastFmUrl(text.substring(startStatIndex, endStatIndex + endSearch.length()).trim()));
        return endStatIndex;
    }

    private static int getImgLoc(String text, LastFmTrackStats lastFmTrackStats, int startImageIndex, boolean fromAmazon) {
        int endImageIndex;
        endImageIndex = text.indexOf("\" alt", startImageIndex);
        LOG.debug("startImageIndex = " + startImageIndex);
        LOG.debug("endImageIndex = " + endImageIndex);
        String imgLoc = text.substring(startImageIndex, endImageIndex).trim();
        lastFmTrackStats.setAlbumImageLocation(imgLoc);
        LOG.info("got image location from " + (fromAmazon ? "Amazon" : "last.fm") + ": " + imgLoc);
        return endImageIndex;
    }

    public static void addTrack(Request request) throws IOException, SAXException, NoSuchAlgorithmException {

        SongInfo songInfo = (SongInfo) request.getSession().getAttribute(Constants.ATTR_SONG_INFO);

        if (songInfo == null) {
            throw new RuntimeException("illegal call encountered: song info does not exist");
        }

        String userName = request.getParamter(Constants.PARAM_LAST_FM_USER_NAME);
        String password = request.getParamter(Constants.PARAM_LAST_FM_PASSWORD);

        LOG.debug("password = " + password);
        LOG.debug("userName = " + userName);

        if (userName == null || password == null) {
            throw new RuntimeException("unable to authendicated to last.fm");
        }

        WebConversation wc = new WebConversation();
        WebRequest req = new GetMethodWebRequest("http://post.audioscrobbler.com");
        req.setParameter("u", userName);
        req.setParameter("c", "mmm");
        req.setParameter("hs", "true");
        req.setParameter("p", "1.1");
        req.setParameter("v", "0.1");
        WebResponse res = wc.getResponse(req);
        String text = res.getText();
        String[] strings = text.split("\n");

        if (strings.length < 3 || strings[0].indexOf("UPTODATE") == -1) {
            String msg = "Unable to authenticate to last.fm";
            LOG.warn("Unable to authenticate to last.fm: " + text);
            throw new RuntimeException(msg);
        }

        String key = strings[1];
        Date date = new Date();

        DateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        req = new GetMethodWebRequest("http://62.216.251.200/protocol_1.1");
        req.setParameter("u", userName);
        String s = md5(md5(password) + key).toLowerCase();
        LOG.debug("hex = " + s);
        req.setParameter("s", s);
        req.setParameter("a[0]", songInfo.getArtist());
        req.setParameter("t[0]", songInfo.getTitle());
        req.setParameter("b[0]", songInfo.getAlbum()!=null?songInfo.getAlbum():"");
        req.setParameter("m[0]", "");
        req.setParameter("l[0]", "255");
        req.setParameter("i[0]", simpleDateFormat.format(date));
        res = wc.getResponse(req);
        text = res.getText();
        if (text.indexOf("OK") == -1) {
            String msg = "a problem was encountered whille attempting to add track to last.fm: " + text;
            LOG.warn(msg);
            throw new RuntimeException(msg);
        }
        LOG.info("successfully added track to last.fm");

    }

    public static String getStationName(String pandoraUserName, String stationId) throws IOException, DocumentException {
        LOG.info("looking up station name for station with id: " + stationId + " user: " + pandoraUserName);
        URL url = new URL("http://feeds.pandora.com/feeds/people/" + pandoraUserName + "/stations.xml?max=100");
        Document document = parse(url);
        Iterator list = document.getRootElement().element("channel").elementIterator("item");

        while (list.hasNext()) {
            Element o = (Element) list.next();
            String value = o.element("link").getText();
            if (value.indexOf(stationId) != -1) {
                String stationName = o.element("title").getText();
                LOG.info("found station name: " + stationName);
                return stationName;
            }
        }
        throw new RuntimeException("station with id: " + stationId + " not found. user: " + pandoraUserName);
    }

    private static Document parse(URL url) throws DocumentException {
        SAXReader reader = new SAXReader();
        return reader.read(url);
    }

    private static String md5(String message) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        MessageDigest md = MessageDigest.getInstance("MD5");
        return hex(md.digest(message.getBytes("CP1252")));
    }

    private static String hex(byte[] array) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < array.length; ++i) {
            sb.append(Integer.toHexString((array[i] & 0xFF) | 0x100).toLowerCase().substring(1, 3));
        }
        String s = sb.toString();
        LOG.debug("s = " + s);
        return s;
    }

}
