package util;

import enums.ReturnCodes;
import enums.FileType;

import java.util.Date;
import java.io.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import properties.DefaultProperties;

public class Util {

    private static Log LOG = LogFactory.getLog(Util.class);

    public static String writeResponseHeader(ReturnCodes returnCode, FileType fileType, String fileName) {
        StringBuilder header = new StringBuilder("HTTP/1.0 ");
        switch (returnCode) {
            case OK:
                header.append("200 OK");
                break;
            case BAD_REQUEST:
                header.append("400 Bad Request");
                break;
            case FORBIDDEN:
                header.append("403 Forbidden");
                break;
            case NOT_FOUND:
                header.append("404 Not Found");
                break;
            case INTERNAL_ERROR:
                header.append("500 Internal server.Server Error");
                break;
            case NOT_IMPLEMENTED:
                header.append("501 Not Implemented");
                break;
        }

        header.append("\r\n");
        header.append(("Date: " + new Date() + "\r\n"));

        switch (fileType) {
            case MP3:
                header.append("Content-Type: application/mp3\r\n");
                if (fileName != null) {
                    LOG.info("setting fileName: " + fileName);
                    header.append("Content-Disposition: attachment; filename=").append(fileName.replace(" ", "_")).append("\r\n");
                }
                break;
            case JPEG:
                header.append("Content-Type: image/jpeg\r\n");
                break;
            case JPG:
                header.append("Content-Type: image/jpeg\r\n");
                break;
            case GIF:
                header.append("Content-Type: image/gif\r\n");
            case ZIP:
                header.append("Content-Type: application/x-zip-compressed\r\n");
            default:
                header.append("Content-Type: text/html\r\n");
                break;
        }
        header.append("Cache-Control: no-cache, no-store, must-revalidate, max-age=-1\r\n");
        header.append("server.Server: " + Constants.APP_NAME + "\r\n");
        header.append("Connection: close\r\n");
        header.append("\r\n");
        return header.toString();
    }

    public static void writeFile(BufferedReader bufferedReader, DataOutputStream output) throws IOException {
        String line;
        while ((line = bufferedReader.readLine()) != null) {
            output.writeBytes(line + "\r\n");
        }
    }

    public static void writeBinary(InputStream fileInputStream, OutputStream output) throws IOException {
        byte[] buf = new byte[32 * 1024];
        int i;
        while ((i = fileInputStream.read(buf)) != -1) {
            output.write(buf, 0, i);
        }
    }

    public static boolean isMultiMedia(FileType fileType) {
        return fileType == FileType.JPEG || fileType == FileType.JPG || fileType == FileType.GIF || fileType == FileType.ZIP;
    }

    public static String addLastFmUrl(String text) {
        return text.replaceAll("href=\"", " target=\"_blank\" href=\"http://last.fm");
    }

    public static File createSongHierarchy(SongInfo songInfo) {
		LOG.info(songInfo);
        File saveDir;
        File stationDir = null;
        File mp3Dir;

        String mp3ArchivePath = DefaultProperties.getMp3ArchivePath();
		LOG.info("Destination path: "+mp3ArchivePath);
        if (mp3ArchivePath != null) {
            File archiveDir = new File(mp3ArchivePath);
            if (!archiveDir.exists()) {
                archiveDir.mkdir();
            }
            mp3Dir = new File(archiveDir, "mp3");
        }
        else {
           mp3Dir = new File("mp3");
        }

        if (!mp3Dir.exists()) {
            mp3Dir.mkdir();
        }

        if (songInfo.getStationName() != null) {
            stationDir = new File(mp3Dir, SafeString.getSafeFileName(songInfo.getStationName()));
            if (!stationDir.exists()) {
                stationDir.mkdir();
            }
        }

        File artistDir = new File(stationDir!=null?stationDir:mp3Dir, SafeString.getSafeFileName(songInfo.getArtist()));
        if (!artistDir.exists()) {
            artistDir.mkdir();
        }
        saveDir = artistDir;

        File albumDir;
        String album = songInfo.getAlbum();
        LOG.info("album = " + album);

        if (album != null) {
            albumDir = new File(artistDir, SafeString.getSafeFileName(album));
            if (!albumDir.exists()) {
                albumDir.mkdir();
            }
            saveDir = albumDir;
        }

        return saveDir;
    }
}
